--------------------------------------------------------------------------------
--
-- Copyright (C) 2004 The Chameleon Team
--
-- This program is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by the Free 
-- Software Foundation; either version 2 of the License, or (at your option) 
-- any later version. This program is distributed in the hope that it will be 
-- useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-- See the GNU General Public License for more details. 
-- You should have received a copy of the GNU General Public License along
-- with this program; if not, write to the Free Software Foundation, Inc., 
-- 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
--
--------------------------------------------------------------------------------
--
-- This module defines the the translation from external AST with 
-- regular exp type/pattern extension to external AST inStandard Haskell 98 
--
--------------------------------------------------------------------------------

module AST.RETrans where

import AST.External 
import AST.Common
import AST.SrcInfo
import Misc
import Misc.ErrorMsg


transProg :: Prog -> Prog
transProg (Prog ms) = Prog (map transModule ms)

transModule :: Module -> Module
transModule (Module id exs ims decs) = 
    let rfs = filter isRegFun decs
    in case rfs of
           [] -> Module id exs ims decs
           _ -> Module id exs ims' decs'
                where 
--                ims' = imports ++ ims
                ims' = ims
                imports | any (==idStr id) reImports = []
		        | otherwise = map mkImport reImports
                mkImport m = let id = mkId builtInSrcInfo m
	                     in  Import id (Qual id) ImAll
                initTenv = annToPreds rfs
                decs' = transDecs rfs (mergeDefs decs) initTenv
-- converts reg exp function signature into the initial type enviornment
annToPreds :: [Dec] -> [Pred]
annToPreds [] = []
annToPreds ((AnnDec src (Sig id Reg (TSchm _ (ArrType src' inr outr)))):adecs) = 
    (Pred src (mkId anonSrcInfo "REEQ") [ (VarType (mkId anonSrcInfo ("t"++(idStr id)++"$")))
                                        , (ArrType src' (rt2st inr) (rt2st outr)) ]):(annToPreds adecs)

                

reImports = ["XHaskell.REInference","XHaskell.RESubtype","XHaskell.RETypes","XHaskell.REtoHT"]


-- arg 1: a list of reg func signatures
-- arg 2: a list of decs
-- arg 3: a set of constraits represting the initial tenv
transDecs :: [Dec] -> [Dec] -> [Pred] -> [Dec] 
transDecs [] decs _ = decs
transDecs (rf:rfs) decs itenv = 
    case rf of
    AnnDec annsrc (Sig id Reg (TSchm (Ctxt preds) (ArrType arrsrc inr outr))) 
        -> let ity = rt2st inr
               oty = rt2st outr
           in 
           case (findValDec id decs) of
           Just dec -> let (tdec,const) = transValDec ity oty dec
                           decs'       = replaceValDec tdec decs
                           iht         = VarType (mkId anonSrcInfo "iht$")
                           oht         = VarType (mkId anonSrcInfo "oht$")
                           re2htpreds  = [(Pred anonSrcInfo (mkId anonSrcInfo "REtoHT") [ity,iht])
                                         ,(Pred anonSrcInfo (mkId anonSrcInfo "REtoHT") [oty,oht])]
                           anndec      = AnnDec annsrc (Sig id Exist (TSchm (Ctxt (itenv++re2htpreds++preds++const)) (ArrType arrsrc iht oht)))
                           decs''      = replaceAnnDec anndec decs'
                       in transDecs rfs decs'' itenv
           Nothing -> transDecs rfs decs itenv
    _ -> bug ("transDecs: Unsupported regular experssion type signature."++(show rf))
    where
    findValDec :: Id -> [Dec] -> Maybe Dec
    findValDec id [] = Nothing
    findValDec id ((dec@(ValDec src (def:defs))):decs) = 
        let (Def id' _ _ _) = def
        in if ((idStr id) == (idStr id')) 
           then Just dec
           else findValDec id decs
    findValDec id (_:decs) =  findValDec id decs
    replaceValDec :: Dec -> [Dec] -> [Dec]
    replaceValDec dec [] = []
    replaceValDec dec1@(ValDec src1 ((Def id1 _ _ _):_)) ((dec2@(ValDec src2 ((Def id2 _ _ _):_))):decs) =
        if ((idStr id1) == (idStr id2)) 
        then (dec1:decs)
        else dec2:(replaceValDec dec1 decs)
    replaceValDec dec1 (dec2:decs) = dec2:(replaceValDec dec1 decs)
    replaceAnnDec :: Dec -> [Dec] -> [Dec]
    replaceAnnDec dec [] = []
    replaceAnnDec (dec1@(AnnDec src1 (Sig id1 _ _ ))) ((dec2@(AnnDec src2 (Sig id2 _ _))):decs) = 
        if ((idStr id1) == (idStr id2))
        then dec1:decs
        else dec2:(replaceAnnDec dec1 decs)
    replaceAnnDec dec1 (dec2:decs) = dec2:(replaceAnnDec dec1 decs)

-- takes input and output (singleton) types, a val dec,
-- returns a val dec and a list of predicates (constraints)
transValDec :: Type -> Type -> Dec -> (Dec,[Pred])
transValDec ity oty (ValDec src (defs@((Def id pat rhs wh):_))) = 
    let (letdecs,preds,tripples) = transDefs 1 ity oty defs
        letexp = LetExp src letdecs (VarExp (mkId src "p$1"))
        (pstys,ptys,gtys) = toSListTy tripples
        pgtys = TupType anonSrcInfo [ptys,gtys]
        pred = Pred anonSrcInfo (mkId anonSrcInfo "XHaskell.REInference.Infer") [ity,pstys,pgtys]
    in ((ValDec src [Def id [(VarPat (mkId src "v$in"))] (RHS1 letexp) []]), (pred:preds))
    where 
    toSListTy :: [(Type,Type,Type)] -> (Type,Type,Type) -- to singleton list type
    toSListTy [] = ( (ConType (mkId anonSrcInfo "XHaskell.RETypes.N"))
                   , (ConType (mkId anonSrcInfo "XHaskell.RETypes.N"))
                   , (ConType (mkId anonSrcInfo "XHaskell.RETypes.N")))
    toSListTy ((ty1,ty2,ty3):rts) = let (tys1,tys2,tys3) = toSListTy rts
                                    in ( (AppType anonSrcInfo 
                                          (AppType anonSrcInfo
                                           (ConType (mkId anonSrcInfo "XHaskell.RETypes.C"))
                                           ty1)
                                          tys1)
                                       , (AppType anonSrcInfo 
                                          (AppType anonSrcInfo
                                           (ConType (mkId anonSrcInfo "XHaskell.RETypes.C"))
                                           ty2)
                                          tys2)
                                       , (AppType anonSrcInfo 
                                          (AppType anonSrcInfo
                                           (ConType (mkId anonSrcInfo "XHaskell.RETypes.C"))
                                           ty3)
                                          tys3)
                                       )
-- additionally returns a list consist of a tripple 
-- Patttern singleton type representation, variable representing pattern precise type and 
-- variable representing pattern r-gamma
transDefs :: Int -> Type -> Type -> [Def] -> ([Dec],[Pred],[(Type,Type,Type)])
transDefs c ity oty [] = ([(ValDec anonSrcInfo [(Def (mkId anonSrcInfo ("p$"++(show c))) [] 
                                                 (RHS1 (AppExp anonSrcInfo (VarExp (mkId anonSrcInfo "Chameleon.Primitive.undefined" -- "error"
										   )) 
                                                                   (LitExp (StrLit anonSrcInfo "NEP.")))) [])])],[],[])
transDefs c ity oty ((Def _ [pat] rhs wh):defs) = 
    let (rest_defs,rest_preds,rest_tripples) = transDefs (c+1) ity oty defs
        pty                    = VarType (mkId anonSrcInfo ("pt$"++(show c))) -- pattern exact type variable
        gty                    = stripg pat                                   -- r-gamma type variable
        pp                     = stripv pat                                   -- bare pattern
--        paty                   = stript pat                                   -- pattern annotated type
        bty                    = VarType (mkId anonSrcInfo ("bt$"++(show c))) -- body type variable
        (bexp,preds)           = transRHS c (show c) bty rhs 
        psty                   = strips pat                                   -- pattern singleton type
    in
    ((ValDec anonSrcInfo [(Def (mkId anonSrcInfo ("p$"++(show c))) [] 
                                   (RHS1 (CaseExp anonSrcInfo 
                                          (mkDCastExp ity pty 
                                           (VarExp (mkId anonSrcInfo "v$in")))
                                          [ (CaseAlt anonSrcInfo 
                                             (ConPat anonSrcInfo (mkId anonSrcInfo "Nothing") []) 
                                             (RHS1 (VarExp (mkId anonSrcInfo ("p$"++(show (c+1)))))) [])
                                          , (CaseAlt anonSrcInfo 
                                             (ConPat anonSrcInfo (mkId anonSrcInfo "Just") 
                                              [VarPat (mkId anonSrcInfo ("pv$"++(show c)))])
                                             (RHS1 (LetExp anonSrcInfo 
                                                    [(PatDec anonSrcInfo 
                                                      (PatBnd pp 
                                                       (RHS1 (mkUCastExp pty gty 
                                                              (VarExp (mkId anonSrcInfo ("pv$"++(show c)))))) []))]
                                                    (mkUCastExp bty oty bexp)))
                                             [])
                                          ]
                                         )
                                   ) []
                          )]):rest_defs,(preds++rest_preds),((psty,pty,gty):rest_tripples))

-- the second parameter is used for generate unique aux type variables
transRHS :: Int -> String -> Type -> RHS -> (Exp,[Pred])
transRHS c s ty (RHS1 exp) = transExp c s ty exp

transExp :: Int -> String -> Type -> Exp -> (Exp,[Pred])
transExp c _ ty (VarExp id) = ((VarExp id),[(Pred anonSrcInfo (mkId anonSrcInfo "REEQ") [ty,(idTy id)])]) 
                                 -- using idTy, by assuming all variables in a function 
                                 -- are unique, need to enforce this in future
transExp c s ty (AppExp si e1 e2) = let s1 = s ++ "1" 
                                        s2 = s ++ "2"
                                        s3 = s ++ "3"
                                        t1 = mkTy c s1
                                        t2 = mkTy c s2
                                        t3 = mkTy c s3
                                        (e1',preds1) = transExp c s1 t1 e1            -- t1 e1 |- (e1',p1')
                                        (e2',preds2) = transExp c s2 t2 e2            -- t2 e2 |- (e2',p2')
                                        preds3 = [(Pred anonSrcInfo 
                                                   (mkId anonSrcInfo "REEQ") 
                                                   [t1,(ArrType anonSrcInfo t3 ty)])] -- t1=t3-> ty
                                    in ((AppExp si e1' (mkUCastExp t2 t3 e2')),(preds1++preds2++preds3))
transExp c s ty (TupExp si [e1,e2]) = let s1 = s ++ "1"
                                          s2 = s ++ "2"
                                          t1 = mkTy c s1
                                          t2 = mkTy c s2
                                          (e1',preds1) = transExp c s1 t1 e1          -- t1 e1 |- (e1',p1')
                                          (e2',preds2) = transExp c s2 t2 e2          -- t2 e2 |- (e2',p2')
                                          preds3 = [(Pred anonSrcInfo
                                                     (mkId anonSrcInfo "REEQ")
                                                     [ty,(TupType anonSrcInfo [t1,t2])])] -- ty = (t1,t2)
                                      in ((TupExp si [e1',e2']),(preds1++preds2++preds3))

-- make up a temp type from a string
mkTy :: Int -> String -> Type
mkTy c s = VarType (mkId anonSrcInfo ("t"++s++"$"++(show c)))

-- make up a type from an id
idTy2 :: Int -> Id -> Type
idTy2 c id = VarType (mkId anonSrcInfo ("t"++(idStr id)++"$"++(show c)))

idTy :: Id -> Type
idTy id = VarType (mkId anonSrcInfo ("t"++(idStr id)++"$"))

-- fixme : currently we ONLY support binary tuples
stripv :: Pat -> Pat 
stripv (AnnPat _ id _) = VarPat id
stripv (TupPat si [p1,p2]) = TupPat si [(stripv p1),(stripv p2)]

stript :: Pat -> Type
stript (AnnPat _ id ty) = rt2st ty
stript (TupPat _ [p1,p2]) = TupType anonSrcInfo [(stript p1),(stript p2)]

stripg :: Pat -> Type
stripg (AnnPat _ id _) = idTy id
stripg (TupPat _ [p1,p2]) = TupType anonSrcInfo [(stripg p1),(stripg p2)]


strips :: Pat -> Type
strips (AnnPat _ id ty) = AppType anonSrcInfo (ConType (mkId anonSrcInfo "VARPat")) (rt2st ty)
strips (TupPat _ [p1,p2]) = AppType anonSrcInfo 
                            (AppType anonSrcInfo 
                             (ConType (mkId anonSrcInfo "PAIRPat")) 
                             (strips p1))
                            (strips p2)


mkDCastExp :: Type -> Type -> Exp -> Exp
mkDCastExp ty1 ty2 v = (AppExp anonSrcInfo 
                        (AppExp anonSrcInfo 
                         (AppExp anonSrcInfo (VarExp (mkId anonSrcInfo "XHaskell.RECast.dcast")) 
                          (AnnExp anonSrcInfo (VarExp (mkId anonSrcInfo "Chameleon.Primitive.undefined")) Exist (TSchm (Ctxt []) ty1)))
                         (AnnExp anonSrcInfo (VarExp (mkId anonSrcInfo "Chameleon.Primitive.undefined")) Exist (TSchm (Ctxt []) ty2)))
                        v)

mkUCastExp :: Type -> Type -> Exp -> Exp
mkUCastExp ty1 ty2 v = (AppExp anonSrcInfo 
                        (AppExp anonSrcInfo 
                         (AppExp anonSrcInfo (VarExp (mkId anonSrcInfo "XHaskell.RECast.ucast")) 
                          (AnnExp anonSrcInfo (VarExp (mkId anonSrcInfo "Chameleon.Primitive.undefined")) Exist (TSchm (Ctxt []) ty1)))
                         (AnnExp anonSrcInfo (VarExp (mkId anonSrcInfo "Chameleon.Primitive.undefined")) Exist (TSchm (Ctxt []) ty2)))
                        v)


-- convert regtype to singleton type representation
-- we keep the source info
rt2st :: Type -> Type
rt2st (StarType src ty) = let star = ConType (mkId src "STAR") 
                              ty'  = rt2st ty
                          in AppType src star ty'
rt2st (OptionType src ty) = let or  = ConType (mkId src "OR")
                                em  = ConType (mkId src "EMPT")
                                opt = AppType src or em
                                ty' = rt2st ty
                            in AppType src opt ty'
rt2st (UnionType src [ty1,ty2]) = let or = ConType (mkId src "OR")
                                      ty1' = rt2st ty1
                                      ty2' = rt2st ty2
                                  in AppType src (AppType src or ty1') ty2'
rt2st (UnionType src (ty:tys)) = let or = ConType (mkId src "OR")
                                     tys' = rt2st (UnionType src tys)
                                     ty' = rt2st ty
                                 in AppType src (AppType src or ty') tys'
rt2st r = r

-- convert regtype to haskell type

isRegFun :: Dec -> Bool
isRegFun (AnnDec _ sig) = isRegSig sig
isRegFun _ = False

isRegSig :: Sig -> Bool
isRegSig (Sig _ Reg _) = True
isRegSig (Sig _ _ _) = False

                                              
